/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.aktien.gui.parts;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CopyOnWriteArraySet;

import org.apache.commons.lang.StringUtils;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.KeyListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import de.frankmuenster.jameica.finanzen.FinanceException;
import de.frankmuenster.jameica.finanzen.FinanceExceptionType;
import de.frankmuenster.jameica.finanzen.StockInfoBean;
import de.frankmuenster.mahoe.gui.utils.SWTFontUtils;
import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.jameica.gui.GUI;
import de.willuhn.jameica.gui.Part;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;

/**
 * Abstrakte Klasse fr eine Wertpapier-Suche
 *
 * @author Frank Mnster
 * @param <I>
 *            das konkrete StockInfoBean fr den jeweiligen Suchanbieter
 */
public abstract class AbstractStockSearchPart<I extends StockInfoBean> implements Part {

    /**
     * Platzhalter im Suchergebnis 6 Zeichen
     */
    protected static final String SPACE_6 = "      "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 10 Zeichen
     */
    protected static final String SPACE_10 = "          "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 20 Zeichen
     */
    protected static final String SPACE_20 = "                    "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 30 Zeichen
     */
    protected static final String SPACE_30 = "                              "; //$NON-NLS-1$

    /**
     * Platzhalter im Suchergebnis 3 Zeichen
     */
    protected static final String SPACE_3 = "   "; //$NON-NLS-1$

    /**
     * Unsere Listeners fr SearchResultSelected
     */
    private final transient Set<SearchResultSelectListener<I>> listeners = new CopyOnWriteArraySet<>();

    /**
     * Liste mit StockInfo's
     */
    protected transient List<I> resultList = new ArrayList<>();

    /**
     * Predefined search text
     */
    private transient String searchText;

    /**
     * Zeilen der Ergebnisliste
     */
    private transient int resultRows;

    /**
     * Suchtext-Eingabefeld
     */
    protected transient Text searchInput;

    /**
     * Die Liste mit den Suchergebnissen
     */
    protected transient org.eclipse.swt.widgets.List searchResult;

    /**
     * Key listeners for the search field
     */
    private AbstractStockSearchPart<I>.SearchInputKeyListener keyListener;

    /**
     * Standardkonstruktor
     *
     * @param listener
     */
    protected AbstractStockSearchPart(final SearchResultSelectListener<I> listener) {

        this(listener, 7);
    }

    /**
     * Standardkonstruktor
     *
     * @param listener
     *            der Lister dem die Suchfeldauswahl angezeigt werden soll.
     * @param resultRows
     */
    protected AbstractStockSearchPart(final SearchResultSelectListener<I> listener, final int resultRows) {

        this(StringUtils.EMPTY, listener, resultRows);
    }

    /**
     * Standardkonstruktor
     *
     * @param searchText
     *            text to search for
     * @param listener
     *            der Lister dem die Suchfeldauswahl angezeigt werden soll.
     * @param resultRows
     */
    protected AbstractStockSearchPart(final String searchText, final SearchResultSelectListener<I> listener,
            final int resultRows) {

        super();
        this.searchText = searchText;
        this.resultRows = resultRows;
        this.addListener(listener);
    }

    /**
     * Fgt eine Listener hinzu
     *
     * @param listener
     *            SearchResultSelectListener der hinzugefgt werden soll
     */
    public final void addListener(final SearchResultSelectListener<I> listener) {

        this.listeners.add(listener);
    }

    /**
     * @see Part#paint(Composite)
     */
    @Override
    public void paint(final Composite parent) throws RemoteException {

        if (parent == null) {
            Logger.error(I18N.tr("AbstractStockSearchPart.logParentIsNull")); //$NON-NLS-1$
            return;
        }
        final GridLayout gridLayout = new GridLayout(2, false);
        parent.setLayout(gridLayout);
        final Label nameLabel = new Label(parent, SWT.NONE);
        nameLabel.setText(I18N.tr("AbstractStockSearchPart.lblName")); //$NON-NLS-1$
        this.searchInput = GUI.getStyleFactory().createText(parent);
        GridData gridData = new GridData();
        gridData.horizontalAlignment = SWT.FILL;
        gridData.grabExcessHorizontalSpace = true;
        this.searchInput.setLayoutData(gridData);
        this.searchInput.setText(this.searchText);
        this.keyListener = new SearchInputKeyListener();
        this.searchInput.addKeyListener(this.keyListener);
        final Label searchResultLabel = new Label(parent, SWT.NONE);
        searchResultLabel.setText(I18N.tr("AbstractStockSearchPart.lblSearchResults")); //$NON-NLS-1$
        gridData = new GridData();
        gridData.verticalAlignment = SWT.BEGINNING;
        searchResultLabel.setLayoutData(gridData);
        this.searchResult = new org.eclipse.swt.widgets.List(parent, SWT.BORDER | SWT.SINGLE | SWT.V_SCROLL);
        gridData = new GridData(GridData.FILL_BOTH);
        gridData.horizontalAlignment = SWT.FILL;
        gridData.grabExcessHorizontalSpace = true;
        gridData.verticalAlignment = SWT.FILL;
        gridData.grabExcessVerticalSpace = true;
        this.searchResult.setLayoutData(gridData);
        Font font = GUI.getDisplay().getSystemFont();
        try {
            font = SWTFontUtils.getMonospacedFont();
        } catch (AssertionError | RuntimeException ex) { // NOPMD
            Logger.warn(I18N.tr("AbstractStockSearchPart.monospacedFontNotFound")); //$NON-NLS-1$
            Application.getMessagingFactory()
                    .sendMessage(new StatusBarMessage(I18N.tr("AbstractStockSearchPart.monospacedFontNotFoundMsg"), //$NON-NLS-1$
                            StatusBarMessage.TYPE_INFO));
        }
        final FontData fontData = font.getFontData()[0];
        fontData.setHeight(8);
        font = new Font(parent.getDisplay(), fontData);
        this.searchResult.setFont(font);
        // Gre festlegen
        for (int i = 0; i < this.resultRows; i++) {
            this.searchResult.add(StringUtils.EMPTY);
        }
        this.searchResult.addSelectionListener(new SeachResultSelectionListener());

        // Start first search if searchText is not empty
        if (StringUtils.isNotEmpty(this.searchText)) {
            this.keyListener.keyReleased(null);
        }
    }

    /**
     * Entfernt einen Listener aus der Liste
     *
     * @param listener
     *            SearchResultSelectListener der entfernt werden soll
     */
    public final void removeListener(final SearchResultSelectListener<I> listener) {

        this.listeners.remove(listener);
    }

    /**
     * Sets the search text and my perform it.
     *
     * @param searchText
     *            the text to search for
     * @param performSearch
     *            start search immediately
     */
    public void setSearchText(final String searchText, final boolean performSearch) {

        this.searchText = searchText;
        if (this.searchInput != null) {
            this.searchInput.setText(searchText);
            if (performSearch) {
                this.keyListener.keyReleased(null);
            }
        }
    }

    /**
     * Formatiert einen Eintrag, damit es nach einer Art Tabelle aussieht.
     *
     * @param bean
     *            der zu formatieren Bean (die Zeile)
     * @return die formatierte Zeile
     */
    protected abstract String formatListEntry(I bean);

    /**
     * Benachrichtigen der registrierten Listeners mit bergabe dem gefundenen
     * QuoteLookupInfoBean Element
     *
     * @param index
     */
    protected void notifyListeners(final int index) {

        if (this.resultList != null && this.resultList.size() > index) {
            for (final SearchResultSelectListener<I> listener : this.listeners) {
                final I bean = this.resultList.get(index);
                listener.searchResultSelected(bean);
            }
        }
    }

    /**
     * Liefert eine Liste mit den StockInfoBean Elementen der Suche.
     *
     * @param search
     *            der Suchstring
     * @return search Liste mit den StockInfoBean Elementen der Suche.
     * @throws FinanceException
     */
    protected abstract List<I> perfromSearch(String search) throws FinanceException;

    /**
     * Listener zum Anzeigen, dass ein ein Suchergebnis angeklickt wurde.
     *
     * @author Frank Mnster
     * @param <I>
     */
    public interface SearchResultSelectListener<I extends StockInfoBean> {

        /**
         * Gibt das angeklickte StockInfo Element zurck
         *
         * @param bean
         */
        void searchResultSelected(I bean);
    }

    /**
     * Auswahllistener der anzeigt, dass ein Suchergebniss angeklickt wurde.
     *
     * @author Frank Mnster
     */
    protected class SeachResultSelectionListener implements SelectionListener {

        /**
         * Standartkonstruktor
         */
        public SeachResultSelectionListener() {

            // absichtlich leer
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetDefaultSelected(org
         *      .eclipse.swt.events.SelectionEvent)
         */
        @Override
        public void widgetDefaultSelected(final SelectionEvent arg0) {

            // nichts zu tun hier
        }

        /**
         * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse
         *      .swt.events.SelectionEvent)
         */
        @Override
        public void widgetSelected(final SelectionEvent arg0) {

            Logger.debug("Item Selected: " //$NON-NLS-1$
                    + Integer.toString(AbstractStockSearchPart.this.searchResult.getSelectionIndex()));
            AbstractStockSearchPart.this.notifyListeners(AbstractStockSearchPart.this.searchResult.getSelectionIndex());
        }
    }

    /**
     * Listener der Tastenanschlge verarbeitet.
     *
     * @author Frank Mnster
     */
    protected class SearchInputKeyListener implements KeyListener {

        /**
         * Minimale Lnge des Suchwortes, bevor Anfragen an Yahoo gesendet
         * werden.
         */
        private static final int MIN_WORD_LENGTH = 3;

        /**
         * Standartkonstruktor
         */
        public SearchInputKeyListener() {

            // absichtlich leer
        }

        /**
         * @see KeyListener#keyPressed(KeyEvent)
         */
        @Override
        public void keyPressed(final KeyEvent arg0) {

            // nichts zu tun hier.
        }

        /**
         * @see KeyListener#keyReleased(KeyEvent)
         */
        @Override
        public void keyReleased(final KeyEvent arg0) {

            final String search = AbstractStockSearchPart.this.searchInput.getText();
            if (search.length() >= SearchInputKeyListener.MIN_WORD_LENGTH) {
                try {
                    AbstractStockSearchPart.this.resultList = AbstractStockSearchPart.this.perfromSearch(search);
                } catch (final FinanceException e) {
                    if (e.getType().equals(FinanceExceptionType.DATA_NOT_FOUND)) {
                        Logger.error(I18N.tr("AbstractStockSearchPart.dataNotFound", e.getMessage())); //$NON-NLS-1$
                        Application.getMessagingFactory()
                                .sendMessage(new StatusBarMessage(I18N.tr("AbstractStockSearchPart.dataNotFound"), //$NON-NLS-1$
                                        StatusBarMessage.TYPE_ERROR));
                    }
                }
                AbstractStockSearchPart.this.searchResult.removeAll();
                if (AbstractStockSearchPart.this.resultList != null) {
                    for (final I stockInfo : AbstractStockSearchPart.this.resultList) {
                        AbstractStockSearchPart.this.searchResult
                                .add(AbstractStockSearchPart.this.formatListEntry(stockInfo));
                    }
                }
            } else {
                AbstractStockSearchPart.this.searchResult.removeAll();
            }
        }
    }

}
