/*******************************************************************************
 * Copyright 2015 - 2021 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.bos.webpages;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang3.tuple.Pair;
import org.openqa.selenium.By;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriverException;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.Select;

import de.frankmuenster.mahoe.banks.bos.web.BosWebAccountInformation;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.AbstractPostBoxPage;
import de.willuhn.logging.Logger;

/**
 * The post box page for the Bank of Scotland
 *
 * @author Frank Mnster
 * @since 2.6.0
 *
 */
public class BosPostBoxPage extends AbstractPostBoxPage<BosWebAccountInformation, WebElement> {

    /**
     * ID of navigation form
     */
    private static final String NAV_FORM = "navForm"; //$NON-NLS-1$

    /**
     * Instantiate a new BosPostBoxPage and loads all documents/announcements
     *
     * @param driver
     *            the Selenium WebDriver with the logged in account
     * @param accountInfo
     *            the BosWebAccountInformation object
     * @throws WebSiteException
     */
    public BosPostBoxPage(final WebDriver driver, final BosWebAccountInformation accountInfo) throws WebSiteException {
        super(driver, accountInfo);

        // Initialise lists
        this.accountSheetDocuments = new ArrayList<>();
        this.otherDocuments = new ArrayList<>();

        // Initialise list positions
        this.accountSheetsPosition = 0;
        this.documentsPosition = 0;

        // open the post box page
        openPostBox();

        // load account sheets
        loadAccountSheets();

    }

    private void loadAccountSheets() throws WebSiteException {

        // new mails
        WebElement navigation = this.driver.findElement(By.id(BosPostBoxPage.NAV_FORM));
        WebElement mails = navigation.findElement(By.linkText("Neu")); //$NON-NLS-1$
        String classText = mails.getAttribute("class");
        if (!classText.contains("active")) {
            Logger.debug("Open New mails");
            Utils.waitIgnoreThreadInterrupts(200);
            mails.click();
        }

        WebElement table = this.driver.findElement(By.id("contentForm")); //$NON-NLS-1$
        if (!table.getText().contains("keine ungelesenen")) { //$NON-NLS-1$

            // TODO copy and paste table read from read mails
            Logger.debug(table.getText());
        }

        // read mails (for transaction sheets only
        navigation = this.driver.findElement(By.id(BosPostBoxPage.NAV_FORM));
        mails = navigation.findElement(By.partialLinkText("Gelesen")); //$NON-NLS-1$
        classText = mails.getAttribute("class");
        if (!classText.contains("active")) {
            Logger.debug("Open read mails");
            Utils.waitIgnoreThreadInterrupts(200);
            mails.click();
        }

        table = this.driver.findElement(By.id("contentForm")); //$NON-NLS-1$
        if (!table.getText().contains("keine ungelesenen")) { //$NON-NLS-1$

            // change sort order
            final Select sortOrder = new Select(this.driver.findElement(By.id("contentForm:inbox-filter-reihenfolge"))); //$NON-NLS-1$
            Logger.debug(sortOrder.getFirstSelectedOption().getText());
            Utils.waitIgnoreThreadInterrupts(1000);
            sortOrder.selectByValue("0"); //$NON-NLS-1$

            // start date
            WebElement startDate = this.driver.findElement(By.id("contentForm:inbox-datetimepicker-start")); //$NON-NLS-1$
            startDate.clear();
            final Calendar cal = Calendar.getInstance();
            cal.set(cal.get(Calendar.YEAR) - 10, 0, 1);
            final String check = Utils.convertLocalDateToString(Utils.dateToLocalDate(cal.getTime()));
            startDate = this.driver.findElement(By.id("contentForm:inbox-datetimepicker-start")); //$NON-NLS-1$
            Utils.waitIgnoreThreadInterrupts(500);
            startDate.sendKeys(check);

            Utils.waitIgnoreThreadInterrupts(500);
            final WebElement show = this.driver.findElement(By.linkText("Anzeigen")); //$NON-NLS-1$
            show.click();

            Utils.waitIgnoreThreadInterrupts(500);

            final WebElement documentTable = this.driver.findElement(By.id("contentForm:transactionList"));
            final List<WebElement> documents = documentTable.findElements(By.tagName("li"));
            for (final WebElement doc : documents) {
                if (doc.findElement(By.xpath("a/div/div[2]")).getText().startsWith("Ihr Kontoauszug zum Tagesgeld")) { //$NON-NLS-2$

                    Logger.debug(doc.getText());

                    final String dateTxt = doc.findElement(By.xpath("a/div/div[1]")).getText(); //$NON-NLS-1$
                    Date date;
                    try {
                        date = Utils.convertTextToDate(dateTxt, "dd.MM.yyyy"); //$NON-NLS-1$
                    } catch (final ParseException e) {
                        throw new WebSiteException("Fehler beim Parsen des Datums",
                                WebSiteExceptionTypes.READ_ACCOUNT_DETAILS_FAILED, e);
                    }

                    // create the pair and add document to list
                    final Pair<Date, WebElement> document = Pair.of(date, doc);
                    this.accountSheetDocuments.add(document);

                }

            }

        }

    }

    private void openPostBox() {

        Logger.debug("Open PostBox"); //$NON-NLS-1$

        if (!elementExist(By.id(BosPostBoxPage.NAV_FORM))) {
            Utils.waitIgnoreThreadInterrupts(5000);
            if (!elementExist(By.id(BosPostBoxPage.NAV_FORM))) {
                Utils.waitIgnoreThreadInterrupts(5000);
                throw new WebDriverException("Postbox konnte nicht geffnet werden."); //$NON-NLS-1$
            }
        }

        final WebElement navigation = this.driver.findElement(By.id(BosPostBoxPage.NAV_FORM));
        final WebElement postBoxLink = navigation.findElement(By.partialLinkText("Postbox")); //$NON-NLS-1$
        Logger.debug(postBoxLink.getAttribute("href"));
        // simply wait a bit
        Utils.waitIgnoreThreadInterrupts(200);
        postBoxLink.click();

        Logger.debug("PostBox open"); //$NON-NLS-1$
    }

}
