/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.web;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.rmi.RemoteException;
import java.time.Duration;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import org.openqa.selenium.By;
import org.openqa.selenium.NoSuchElementException;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.banks.santander.clearing.ClearingEntry;
import de.frankmuenster.mahoe.banks.santander.pdfextractor.SantanderPdfExtractor;
import de.frankmuenster.mahoe.banks.santander.webpages.SantanderDetailsPage;
import de.frankmuenster.mahoe.banks.santander.webpages.SantanderLoginPage;
import de.frankmuenster.mahoe.banks.santander.webpages.SantanderPostBoxPage;
import de.frankmuenster.mahoe.banks.santander.webpages.SantanderTransactionsPage;
import de.frankmuenster.mahoe.pdfextractor.InvalidCardNumberException;
import de.frankmuenster.mahoe.pdfextractor.PdfBuchung;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.PasswdCache;
import de.frankmuenster.mahoe.utils.TransactionSaver;
import de.frankmuenster.mahoe.utils.WebUtilsSelenium;
import de.frankmuenster.mahoe.web.AbstractSeleniumWebSite;
import de.frankmuenster.mahoe.web.AbstractWebSite;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebAccountInformation;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.DetailsPage;
import de.frankmuenster.mahoe.web.webpages.LoginPage;
import de.frankmuenster.mahoe.web.webpages.TransactionsPage;
import de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.KontoType;
import de.willuhn.jameica.hbci.rmi.Umsatz;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Santander Bank Website.
 *
 * @author Frank Mnster
 * @param <I>
 *            SantanderWebAccountInformation
 * @param <M>
 *            SantanderBankMessages
 */
@SuppressWarnings({ "PMD.ExcessiveImports", "PMD.TooManyMethods" })
public class SantanderWebSite extends AbstractSeleniumWebSite<SantanderWebAccountInformation, SantanderBankMessages> {

    /** Class name of the loading overlay div */
    protected static final String CLASS_NAME_FOR_LOADING_OVERLAY = "div[class^=LoadingOverlay__overlay]"; //$NON-NLS-1$

    /** I18n translation string */
    private static final String I18N_ERROR_SETTING_SALDO = "SantanderWebSite.errorSettingSaldo"; //$NON-NLS-1$

    /** Expected loginParams */
    private static final int EXPECTED_LOGIN_PARAMS = 1;

    /** The SantanderWebSite instance */
    private static SantanderWebSite webSite;

    /** Base URL of the logged-in account */
    private String baseUrl;

    /** the clearing entry */
    private ClearingEntry clearingEntry;

    /**
     * Indicates if that is the first PDF to create the clearance transaction if
     * yes
     */
    private boolean firstPDF;

    /** account type */
    private final KontoType type;

    /** The account login */
    private final String userName;

    /**
     * Returns the account sheet as byte array
     *
     * @param document
     *            the message from the postbox with download link inside
     * @param webClient
     * @return the account sheet as byte array
     * @throws WebSiteException
     *             if the download of the account sheet failed
     */
    public static byte[] getAccountSeleniumSheetAsByteArray(final Pair<Date, WebElement> document,
            final WebDriver webClient) throws WebSiteException {

        // extract link element and download as byte array
        Logger.debug(document.getKey().toString() + ":" + document.getValue().getText()); //$NON-NLS-1$
        // final WebElement pdflink = document.getValue()
        // .findElement(By.className("postboxDateWrapper__listWrapper___2fHdq"));
        // //$NON-NLS-1$

        // scroll into view
        WebUtilsSelenium.scrollTo(webClient, document.getValue());

        // wait for any overlay
        SantanderWebSite.waitForLoadingOverlay(webClient);

        byte[] pdf = WebUtilsSelenium.getSeleniumPdfAsByteArray(webClient, document.getValue());
        // check if file has content
        if (pdf.length == 0) {
            // Retry download
            pdf = WebUtilsSelenium.getSeleniumPdfAsByteArray(webClient, document.getValue());
        }
        if (pdf.length == 0) {
            throw new WebSiteException(I18N.tr("SantanderWebSite.atLeastOneDownloadFailed"), //$NON-NLS-1$
                    WebSiteExceptionTypes.FILE_DOWNLOAD_FAILED);
        }

        return pdf;

    }

    /**
     * Liefert die Instance der WebSeite.
     *
     * @param konto
     *            the konto
     * @return die WebSeite
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     */
    public static SantanderWebSite getInstance(final Konto konto) throws WebSiteException, ApplicationException {

        String userName = null;
        KontoType type = null;
        try {
            userName = konto.getKundennummer();
            type = KontoType.find(konto.getAccountType());
            // IBAN only needed for TagesGeld
            if (KontoType.GIRO.equals(type) && StringUtils.isEmpty(konto.getIban())) {
                throw new WebSiteException(I18N.tr("SantanderWebSite.ibanIsMissing"), //$NON-NLS-1$
                        WebSiteExceptionTypes.ACCOUNT_INFO_WRONG);
            }
        } catch (final RemoteException e) {

            final String msg = I18N.tr("SantanderWebSite.dbError"); //$NON-NLS-1$
            Logger.error(msg, e);
            throw new ApplicationException(msg, e);
        }

        if (SantanderWebSite.webSite == null) {
            SantanderWebSite.webSite = new SantanderWebSite(konto, type, userName);
        } else {
            // Ist der Benutzername und der Kontotyp derselbe
            if (SantanderWebSite.webSite.userName.equals(userName) && SantanderWebSite.webSite.isSupporting(type)) {

                // Update the accountInfo object
                SantanderWebSite.webSite.setAccountBasicInfo(konto);

                // call getWebclient to be sure Selenium Brwoser is up and
                // running
                SantanderWebSite.webSite.getWebClient();
            } else {
                // first logoff
                SantanderWebSite.webSite.logoff();
                // then create a new instance
                SantanderWebSite.webSite = new SantanderWebSite(konto, type, userName);
            }
        }
        return SantanderWebSite.webSite;
    }

    /**
     * Waits until the loading overlay
     * <code>LoadingOverlay__overlay___GvaFv</code> is invisible.
     *
     * @param driver
     *            with the current page
     */
    public static void waitForLoadingOverlay(final WebDriver driver) {

        // The waiter to find the elements
        final WebDriverWait wait = new WebDriverWait(driver, Duration.ofSeconds(30));

        final List<WebElement> overLays = driver
                .findElements(By.cssSelector(SantanderWebSite.CLASS_NAME_FOR_LOADING_OVERLAY));
        if (!overLays.isEmpty()) {
            final WebElement overLay = overLays.get(0);
            wait.until(ExpectedConditions.invisibilityOf(overLay));
        }
    }

    /**
     * Erstellt ein neues Webseiten-Objekt zur Kontaktaufnahme mit der BCEE.
     *
     * @param konto
     *            the konto
     * @param type
     * @param userName
     *            the user name
     * @throws WebSiteException
     *             the web site exception
     */
    protected SantanderWebSite(final Konto konto, final KontoType type, final String userName) throws WebSiteException {

        super(konto);

        this.userName = userName;
        this.type = type;

    }

    /**
     * Creates the sepa transfer.
     *
     * @param aUeberweisung
     *            the a ueberweisung
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#createSepaTransfer(de.willuhn.jameica.hbci.rmi.AuslandsUeberweisung)
     */
    @Override
    public void createSepaTransfer(final AuslandsUeberweisung aUeberweisung)
            throws WebSiteException, ApplicationException {

        Logger.info(I18N.tr("AbstractWebSite.transferNotSupported")); //$NON-NLS-1$
    }

    /**
     * Gets the account information.
     *
     * @return the account information
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getAccountInformation()
     */
    @Override
    public SantanderWebAccountInformation getAccountInformation() throws WebSiteException, ApplicationException {

        // Add iban to account info

        final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$

        if (KontoType.KREDITKARTE.equals(this.type)) {

            final DetailsPage details = new SantanderDetailsPage(this.webClient, this.baseUrl, this.accountInfo);

            // saldo not tested to pass the Exception as salso is needed
            this.accountInfo.setSaldo(details.getSaldo());

            try {
                // available amount
                this.accountInfo.setSaldoAvailable(details.getAvailableAmount());

                final StringBuilder comment = new StringBuilder();
                comment.append("Limit: ").append(details.getCreditLimit()) //$NON-NLS-1$
                        .append("\nLetzte Abrechnung: ").append(details.getLastTransaction().format(formatter)) //$NON-NLS-1$
                        .append("\nFlligkeit nchste Lastschrift: ") //$NON-NLS-1$
                        .append(details.getNextSubtraction().format(formatter)).append("\nNchste Kartenabrechnung: ") //$NON-NLS-1$
                        .append(details.getNextBill().format(formatter)).append("\nKontoname: ") //$NON-NLS-1$
                        .append(details.getProductDescription());

                this.accountInfo.setComment(comment.toString());
            } catch (final WebSiteException e) {
                // Only log as info is only nice to have
                Logger.error(I18N.tr("SantanderWebSite.errorGetAccountDetail"), e); //$NON-NLS-1$
            }

        } else {
            try {
                this.accountInfo.setIban(this.accountInfo.getKonto().getIban());
            } catch (final RemoteException e) {
                throw new WebSiteException(I18N.tr("SantanderWebSite.ibanIsMissing"), //$NON-NLS-1$
                        WebSiteExceptionTypes.ACCOUNT_INFO_WRONG, e);
            }
            // details page
            final DetailsPage details = new SantanderDetailsPage(this.webClient, this.baseUrl, this.accountInfo);
            // get saldo
            this.accountInfo.setSaldo(details.getSaldo());

        }

        return this.accountInfo;

    }

    /**
     * @see AbstractWebSite#getAccountSheets(WebAccountInformation)
     */
    @Override
    public List<MahoeKontoauszugPDF> getAccountSheets(final SantanderWebAccountInformation accountInfo)
            throws WebSiteException {

        final SantanderAccountStatementFinder accountSheets = new SantanderAccountStatementFinder(this.webClient,
                accountInfo);
        accountSheets.findStatements();

        return accountSheets.getSheets();

    }

    /**
     * Gets the booked transactions.
     *
     * @return the booked transactions
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getBookedTransactions()
     */
    @Override
    public List<Umsatz> getBookedTransactions() throws WebSiteException, ApplicationException {

        // create empty transaction list
        final ArrayList<Umsatz> transactions = new ArrayList<>();

        // select correct account type
        if (KontoType.KREDITKARTE.equals(this.type)) {
            // reset clearance indicator
            this.firstPDF = true;

            try {
                // get the credit card number
                final String cardNumber = this.accountInfo.getKonto().getMeta(MahoeSynchronizeBackend.CARD_NUMBER,
                        StringUtils.EMPTY);
                // retrive booked transaction for this card
                transactions.addAll(getCreditCardBooked(cardNumber));

            } catch (final RemoteException e) {
                Logger.error(I18N.tr("SantanderWebSite.errorReadingBookedCreditcardTransactions"), e); //$NON-NLS-1$
                throw new ApplicationException(I18N.tr("SantanderWebSite.errorReadingBookedCreditcardTransactions"), e); //$NON-NLS-1$
            }

        } else {
            // load of transactions on the web page
            try {
                // open the account page
                final TransactionsPage accountpage = new SantanderTransactionsPage(this.webClient, this.baseUrl,
                        this.accountInfo);

                // retrieve all transactions from the account page
                transactions.addAll(accountpage.getTransactions(this.accountInfo.getLastTransactionDate()));

                // Add Saldo
                addSaldo(transactions);

            } catch (final WebSiteException e) {
                Logger.error(I18N.tr("SantanderWebSite.errorReadingBookedGiroTransactions"), e); //$NON-NLS-1$
                throw new ApplicationException(I18N.tr("SantanderWebSite.errorReadingBookedGiroTransactions"), e); //$NON-NLS-1$
            }
        }
        return transactions;
    }

    /**
     * Returns the clearing transaction
     *
     * @return the clearing transaction
     */
    public ClearingEntry getClearingEntry() {

        return this.clearingEntry;
    }

    /**
     * Gets the pre booked transactions.
     *
     * @return the pre booked transactions
     * @throws WebSiteException
     *             the web site exception
     * @throws ApplicationException
     *             the application exception
     * @see de.frankmuenster.mahoe.web.WebSite#getPreBookedTransactions()
     */
    @Override
    public List<Umsatz> getPreBookedTransactions() throws WebSiteException, ApplicationException {

        final ArrayList<Umsatz> transactions = new ArrayList<>();

        if (KontoType.KREDITKARTE.equals(this.type)) {

            try {

                final TransactionsPage accountpage = new SantanderTransactionsPage(this.webClient, this.baseUrl,
                        this.accountInfo);

                transactions.addAll(accountpage.getTransactions(this.accountInfo.getLastTransactionDate()));

                // Add saldo
                addSaldo(transactions);

            } catch (final WebSiteException e) {
                Logger.error(I18N.tr("SantanderWebSite.errorReadingPreBookedCreditcardTransactions"), e); //$NON-NLS-1$
                throw new ApplicationException(I18N.tr("SantanderWebSite.errorReadingPreBookedCreditcardTransactions"), //$NON-NLS-1$
                        e);
            }
        }
        return transactions;
    }

    /**
     * Checks if is still logged in.
     *
     * @return true, if is still logged in
     * @throws WebSiteException
     *             the web site exception
     * @see de.frankmuenster.mahoe.web.WebSite#isStillLoggedIn()
     */
    @Override
    public boolean isStillLoggedIn() throws WebSiteException {

        boolean isStillLoggedIn = false;

        if (new SantanderLoginPage(this.webClient).isLoggedIn()) {
            // Login successful set base URL
            final String actUrl = this.webClient.getCurrentUrl();
            this.baseUrl = actUrl.substring(0, actUrl.indexOf('#') + 1);
            isStillLoggedIn = true;
        }

        return isStillLoggedIn;

    }

    /**
     * Login.
     *
     * @param password
     *            the password
     * @param loginParams
     *            muss Login_URL enthalten
     * @throws WebSiteException
     *             the web site exception
     * @see de.frankmuenster.mahoe.web.WebSite#login(char[], java.lang.String[])
     */
    @Override
    public void login(final char[] password, final String... loginParams) throws WebSiteException {

        if (loginParams.length != SantanderWebSite.EXPECTED_LOGIN_PARAMS) {
            throw new WebSiteException(I18N.tr("AbstractWebSite.loginParameterNumberWrong"), //$NON-NLS-1$
                    WebSiteExceptionTypes.LOGIN_ERROR);
        }

        try {
            final LoginPage loginPage = new SantanderLoginPage(this.webClient);
            loginPage.login(this.userName, password);
        } catch (final WebSiteException e) {
            if (e.getType().equals(WebSiteExceptionTypes.LOGIN_FAILED)
                    || e.getType().equals(WebSiteExceptionTypes.LOGIN_ERROR)) {
                // Passwort zurcksetzen
                PasswdCache.revoke(SantanderBank.WALLET_PREFIX, loginParams[0]);
            }
            Arrays.fill(password, ' ');
            throw e;
        }
        Arrays.fill(password, ' ');

        // Login successful set base URL
        final String actUrl = this.webClient.getCurrentUrl();
        this.baseUrl = actUrl.substring(0, actUrl.indexOf('#') + 1);

    }

    /**
     * Logout
     *
     * @throws WebSiteException
     *             the web site exception
     * @see de.frankmuenster.mahoe.web.WebSite#logoff()
     */
    @Override
    public void logoff() throws WebSiteException {

        if (!isStillLoggedIn()) {
            return;
        }

        try {

            SantanderWebSite.waitForLoadingOverlay(this.webClient);

            // The waiter to find the elements
            final WebDriverWait wait = new WebDriverWait(this.webClient, Duration.ofSeconds(30));
            wait.until(ExpectedConditions
                    .elementToBeClickable(By.cssSelector("span[class^=SBHeaderMenuLinkLogout__SVGWrapper"))); //$NON-NLS-1$

            // select logout button
            final WebElement logoffBtn = this.webClient
                    .findElement(By.cssSelector("span[class^=SBHeaderMenuLinkLogout__SVGWrapper")); //$NON-NLS-1$
            // scroll into view and click
            WebUtilsSelenium.scrollTo(this.webClient, logoffBtn);
            wait.until(ExpectedConditions.elementToBeClickable(logoffBtn));
            SantanderWebSite.waitForLoadingOverlay(this.webClient);

            try {
                logoffBtn.click();
            } catch (final Exception e) { // NOPMD on 01.02.23, 18:59
                SantanderWebSite.waitForLoadingOverlay(this.webClient);
                logoffBtn.click();
            }

            if (!this.webClient.findElement(By.tagName("body")).getText() //$NON-NLS-1$
                    .contains(I18N.tr("SantanderWebSite.logoffSuccessful"))) { //$NON-NLS-1$
                Logger.error(I18N.tr("AbstractWebSite.logoutFailed")); //$NON-NLS-1$
            }

        } catch (final NoSuchElementException e) {
            // Only log
            Logger.error(I18N.tr("AbstractWebSite.logoutFailed"), e); //$NON-NLS-1$

        }

        // finally save the cookies
        saveCookies(SantanderBank.COOKIE_FILE);

    }

    /**
     * Gets the startdate.
     *
     * @return the startdate
     * @see de.frankmuenster.mahoe.web.AbstractWebSite#getStartdate()
     */
    @Override
    protected Calendar getStartdate() {

        // Setzte das startDatum auf jetzt minus 360 Tage
        final Calendar startCalendar = Calendar.getInstance();
        startCalendar.add(Calendar.DAY_OF_YEAR, -360);
        return startCalendar;
    }

    /**
     * &Uuml;perf&uuml;pt, ob die WebSeiteninstance den Kontotypen
     * unterst&uuml;tzt.
     *
     * @param type
     *            der Kontoyp
     * @return <i>true</i>, wenn der Kontotyp unterst&uuml;tzt wird,
     *         <i>false</i> sonst.
     */
    protected boolean isSupporting(final KontoType type) {

        return KontoType.GIRO.equals(type) || KontoType.KREDITKARTE.equals(type);
    }

    /**
     * Add the saldo to the transactions in the list. the list must be ordered
     * from youngest to oldest and the saldo in the accountInfo needs to be
     * up-to-date.
     *
     * @param transactions
     *            list with transactions to add the saldo too.
     * @throws ApplicationException
     *             in case the actual amount could not be read from or the saldo
     *             could not be written to the transactions.
     */
    private void addSaldo(final List<Umsatz> transactions) throws ApplicationException {

        try {
            // Add saldo
            BigDecimal saldo = this.accountInfo.getSaldo().setScale(2);
            for (final Umsatz transaction : transactions) {
                // set saldo
                transaction.setSaldo(saldo.doubleValue());
                // calculate for next transaction
                saldo = saldo.subtract(new BigDecimal(transaction.getBetrag()).setScale(2, RoundingMode.HALF_UP)); // NOPMD
            }

        } catch (final RemoteException e) {
            Logger.error(I18N.tr(SantanderWebSite.I18N_ERROR_SETTING_SALDO), e);
            throw new ApplicationException(I18N.tr(SantanderWebSite.I18N_ERROR_SETTING_SALDO), e);
        }
    }

    /**
     * Liefert eine Ausgleichbuchung aus den Daten des Auszuges
     *
     * @param extractor
     *            der PDFExtraktor
     */
    private void createClearingEntry(final SantanderPdfExtractor extractor) {

        // Ausgleichsbuchung anlegen, wenn erster Kontoauszug
        if (this.firstPDF) {
            this.clearingEntry = extractor.getAusgleichsBuchung();
            this.firstPDF = false;
        }
    }

    /**
     * Read all new transactions from the account sheets until
     * accountInfo.lastTransactionDate is reached
     *
     * @param cardNumber
     *
     * @return a list with new transactions
     * @throws WebSiteException
     * @throws RemoteException
     */
    private List<Umsatz> getCreditCardBooked(final String cardNumber) throws RemoteException, WebSiteException {

        final List<Umsatz> transactions = new ArrayList<>();

        // load transactions from PDFs
        final SantanderPostBoxPage postBoxPage = new SantanderPostBoxPage(this.webClient, this.accountInfo);

        postBoxPage.resetAccountSheetReadPosition();
        while (postBoxPage.hasNextAccountSheet()) {
            try {
                // get the next document link
                final Pair<Date, WebElement> document = postBoxPage.nextAccountSheet();

                // check date
                if (document.getKey().before(this.accountInfo.getLastTransactionDate())) {
                    Logger.debug(I18N.tr("TargoBankWebSiteTransactionsImpl.reachedLastPdf")); //$NON-NLS-1$
                    break;
                }
                // get the PDF as byte array
                final byte[] pdf = SantanderWebSite.getAccountSeleniumSheetAsByteArray(document, this.webClient);

                // read Transactions from PDF
                final SantanderPdfExtractor extractor = new SantanderPdfExtractor(pdf, cardNumber); // NOPMD
                Logger.debug(I18N.tr("AbstractWebSite.extractTransactions", cardNumber)); //$NON-NLS-1$
                final List<PdfBuchung> buchungenPDF = extractor.getExtract();
                // create clearance entry if first PDF
                this.createClearingEntry(extractor);

                // Add all transactions
                transactions.addAll(TransactionSaver.getTransactions(buchungenPDF));

            } catch (final InvalidCardNumberException e) {
                Logger.warn(I18N.tr("SantanderWebSite.notMatchingCreditcradNumber") + e.getLocalizedMessage()); //$NON-NLS-1$
            }
        }

        return transactions;

    }

    /**
     * An Enum for the existing document types in the document box
     *
     * @author Frank Mnster
     */
    public enum DocumentType {

        /**
         * Credit card billing
         */
        KARTENABRECHNUNG("Kartenabrechnung"), //$NON-NLS-1$

        /**
         * Back account statement
         */
        KONTOAUSZUG("Kontoauszug"); //$NON-NLS-1$

        private String value;

        DocumentType(final String value) { // NOPMD

            this.value = value;
        }

        /**
         * @return the value
         */
        public String getValue() {

            return this.value;
        }

    }
}
