/*******************************************************************************
 * Copyright 2015 - 2020 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.santander.webpages;

import java.time.Duration;
import java.util.List;

import org.openqa.selenium.By;
import org.openqa.selenium.JavascriptExecutor;
import org.openqa.selenium.WebDriver;
import org.openqa.selenium.WebDriverException;
import org.openqa.selenium.WebElement;
import org.openqa.selenium.support.ui.ExpectedConditions;
import org.openqa.selenium.support.ui.WebDriverWait;

import de.frankmuenster.mahoe.banks.santander.SantanderBank;
import de.frankmuenster.mahoe.banks.santander.web.SantanderWebSite;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.SecureCharSequence;
import de.frankmuenster.mahoe.utils.SeleniumCookie;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.utils.WebUtilsSelenium;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.frankmuenster.mahoe.web.WebSiteExceptionTypes;
import de.frankmuenster.mahoe.web.webpages.LoginPage;
import de.frankmuenster.mahoe.web.webpages.SeleniumWebPage;
import de.willuhn.logging.Logger;

/**
 * The login page for the Santander Bank
 *
 * @author Frank Mnster
 * @since 2.4.0
 *
 */
public class SantanderLoginPage extends SeleniumWebPage implements LoginPage {

    /**
     * String for user id field
     */
    private static final String USER_ID = "alias"; //$NON-NLS-1$

    /**
     * String for password field id
     */
    private static final String PASSWORD_ID = "password2"; //$NON-NLS-1$

    /**
     * Creates a new LoginPage for the Santander Bank
     *
     * @param driver
     *            the {@link WebDriver} to use for that page
     */
    public SantanderLoginPage(final WebDriver driver) {
        super(driver);
    }

    /**
     * Check if a user is logged in. There is no check if a specific account is
     * logged in.
     *
     * @return <code>true</code> if an user is logged in, <code>false</code>
     *         else.
     */
    @Override
    public boolean isLoggedIn() {

        boolean isLoggedIn = false;
        try {

            // first test if we are on the correct bank
            final String url = this.driver.getCurrentUrl();
            if (url.contains("santander.de")) { //$NON-NLS-1$
                final String check = this.driver.getPageSource();
                if (check.contains("Abmelden")) { //$NON-NLS-1$
                    isLoggedIn = true;
                    // Fall back to home page
                    this.driver.get(SantanderBank.LOGIN_URL + "/#/"); //$NON-NLS-1$
                    // wait for loading is finished
                    SantanderWebSite.waitForLoadingOverlay(this.driver);
                }
            }
        } catch (final WebDriverException e) {
            // Only log
            Logger.debug("Selenium session already quited." + e.getLocalizedMessage()); //$NON-NLS-1$
        }

        return isLoggedIn;

    }

    /**
     * Logging the given user.
     *
     * @param login
     *            the account to login
     * @param password
     *            the password
     * @throws WebSiteException
     *             if the login fails
     */
    @Override
    public void login(final String login, final char[] password) throws WebSiteException { // NOPMD

        // check if not already logged-in
        if (!isLoggedIn()) {

            // open login page
            final WebDriverWait waitLogin = new WebDriverWait(this.driver, Duration.ofSeconds(60));
            this.driver.get(SantanderBank.LOGIN_URL);
            waitLogin.until(ExpectedConditions.visibilityOfElementLocated(By.id(SantanderLoginPage.USER_ID)));

            // reload cookies
            SeleniumCookie.read(this.driver, SantanderBank.COOKIE_FILE);
            this.driver.get(SantanderBank.LOGIN_URL);
            waitLogin.until(ExpectedConditions.visibilityOfElementLocated(By.id(SantanderLoginPage.USER_ID)));

            // check for cookie acceptance button
            checkCookieWarning();

            // search for the submit button
            final WebElement submitBtn = getSubmmitButton();

            // is submit button found
            if (submitBtn == null) {
                Logger.error(I18N.tr("LoginPage.loginBtnNotFound")); //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("LoginPage.loginBtnNotFound"), WebSiteExceptionTypes.LOGIN_ERROR); //$NON-NLS-1$
            }

            // get fields for alias and password
            final WebElement alias = this.driver.findElement(By.id(SantanderLoginPage.USER_ID));
            final WebElement passwd = this.driver.findElement(By.id(SantanderLoginPage.PASSWORD_ID));

            // Set alias
            Logger.debug("Setzte Kundenummer auf: ****" + login.substring(login.length() - 4)); //$NON-NLS-1$
            alias.sendKeys(login);
            // Set password

            final SecureCharSequence securePW = new SecureCharSequence(password);
            passwd.sendKeys(securePW);
            securePW.wipe();

            // Scroll and click on submit login form
            WebUtilsSelenium.scrollTo(this.driver, submitBtn, false);
            submitBtn.click();

            // wait for the overlay to be removed
            SantanderWebSite.waitForLoadingOverlay(this.driver);

            // wait until the logout button is found
            final WebDriverWait waitTan = new WebDriverWait(this.driver, Duration.ofSeconds(120));
            waitTan.until(ExpectedConditions.or(
                    ExpectedConditions
                            .presenceOfElementLocated(By.className("SBHeaderMenuLinkLogout__SVGWrapper___zkwCR")), //$NON-NLS-1$
                    ExpectedConditions
                            .presenceOfElementLocated(By.className("LoginForm__loginFormAlertError___tehlq")))); //$NON-NLS-1$

            // wait for the overlay to be removed
            SantanderWebSite.waitForLoadingOverlay(this.driver);

            // re-check for cookies - sometimes they are slow
            checkCookieWarning();

            // do a final check, if login was successful
            final String check = this.driver.getPageSource();

            // Logger.debug(check);

            if (check.contains("Ihr Login war nicht erfolgreich.")) { //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("Login.wrongCredentials"), WebSiteExceptionTypes.LOGIN_FAILED); //$NON-NLS-1$
            }
            if (!check.contains("Finanzbersicht")) { //$NON-NLS-1$
                throw new WebSiteException(I18N.tr("TargoBankWebSite.loginFailed"), WebSiteExceptionTypes.LOGIN_ERROR); //$NON-NLS-1$
            }

        }
    }

    /**
     * Check if the cookie warning is displayed.If yes the acceptance button is
     * clicked.
     *
     * @param driver
     *            the {@link WebDriver} with the page (login?) to check for
     *            cookie warning
     */
    private void checkCookieWarning() {

        Utils.waitIgnoreThreadInterrupts(2000);

        final WebElement shadowHost = this.driver.findElement(By.id("usercentrics-root")); //$NON-NLS-1$

        // get the shadow root as the popup will be shown in there
        final WebElement ele = (WebElement) ((JavascriptExecutor) this.driver)
                .executeScript("return arguments[0].shadowRoot.children[0]", shadowHost); //$NON-NLS-1$

        final List<WebElement> cookieBtn = ele.findElements(By.xpath("//*[@data-testid='uc-accept-all-button']")); //$NON-NLS-1$
        if (!cookieBtn.isEmpty()) {
            // Cookie banner is visible -- click button
            cookieBtn.get(0).click();
        }
    }

    /**
     * Search for a button-element with the text "Einloggen".
     *
     * @param driver
     *            the {@link WebDriver} with a page loaded to search for the
     *            element
     * @return the element if found, <i>null</i> else
     */
    private WebElement getSubmmitButton() {

        // Initialise the element with NULL
        WebElement submitBtn = null;
        // get all button-elements
        final List<WebElement> buttons = this.driver.findElements(By.tagName("button")); //$NON-NLS-1$
        // Loop over all elements
        for (final WebElement webElement : buttons) {
            // extract the text from the element
            final String text = webElement.getText().trim();
            Logger.debug(text);
            // Check if text is "Einloggen"
            if ("Einloggen".equalsIgnoreCase(text)) { //$NON-NLS-1$
                // assign to returning element
                submitBtn = webElement;
                // leave the loop as we have found the element
                break;
            }
        }
        // return the element
        return submitBtn;
    }

}
