/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.banks.targobank.pdfextractor;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;
import java.util.Locale;

import org.apache.commons.lang.StringUtils;
import org.apache.pdfbox.text.PDFTextStripper;

import de.frankmuenster.mahoe.pdfextractor.AbstractBankPDFPage;
import de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor;
import de.frankmuenster.mahoe.pdfextractor.AbstractTransactionCreator;
import de.frankmuenster.mahoe.pdfextractor.PdfBuchung;
import de.frankmuenster.mahoe.pdfextractor.PdfEintrag;
import de.frankmuenster.mahoe.pdfextractor.PdfExtractorException;
import de.frankmuenster.mahoe.pdfextractor.PdfExtractorExceptionType;
import de.frankmuenster.mahoe.utils.Utils;
import de.willuhn.logging.Logger;

/**
 * Class for extracting entry information from a TargoBank PDF bank statement.
 *
 * @author Frank Mnster
 */
public class TargoBankPdfExtractor extends AbstractPdfExtractor {

    /**
     * Der Text der die letzte Seite mit Umstzen angibt
     */
    private static final String LAST_PAGE_TEXT = "Neuer Saldo:"; //$NON-NLS-1$

    /**
     * Zeilenhhe der Buchungszeile im MahoeMain PDF.
     */
    private static final double LINE_HEIGH = 9.35;

    /**
     * Dezimalformatierer
     */
    private static final NumberFormat N_FORMAT = NumberFormat.getInstance(Locale.GERMAN);

    /**
     * Mask card number to pass check
     */
    private static String maskCardNumber(final String cardnumber) {

        final String cleaned = StringUtils.deleteWhitespace(cardnumber);
        final String masked = cleaned.substring(0, 6) + "XXXXXX" + cleaned.substring(12, 16); //$NON-NLS-1$
        Logger.debug(String.format("Masked number : %s", masked)); //$NON-NLS-1$

        return masked;

    }

    /**
     * Constructs a new PDF-Extraktor for TargoBank based on byte array
     *
     * @see AbstractPdfExtractor#AbstractPdfExtractor(byte[], String)
     */
    public TargoBankPdfExtractor(final byte[] byteArray, final String kartenNummer) {

        super(byteArray, TargoBankPdfExtractor.maskCardNumber(kartenNummer));
    }

    /**
     * Erstellt einen PDF-Extraktor der TargoBank
     *
     * @see AbstractPdfExtractor#AbstractPdfExtractor(InputStream, String)
     */
    public TargoBankPdfExtractor(final InputStream inputStream, final String kartenNummer) {

        super(inputStream, TargoBankPdfExtractor.maskCardNumber(kartenNummer));
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#addYear
     */
    @Override
    protected void addYear(final List<PdfEintrag> entries) {

        // change year to four digits
        final StringBuilder date = new StringBuilder(10);
        for (final PdfEintrag entry : entries) {
            // Bei Partnerkarten kommt es zu fehlerhaften Eintrgen, die hier zu
            // einer Exception fhren
            try {
                date.append(entry.getWort().substring(0, 6)).append("20").append(entry.getWort().substring(6, 8)); //$NON-NLS-1$
                entry.setWort(date.toString());
                date.setLength(0);
            } catch (final NumberFormatException e) { // NOPMD
                // ignorieren wir
            }
        }
    }

    /**
     * @throws IOException
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfPageEven()
     */
    @Override
    protected AbstractBankPDFPage getBankPdfPageEven() throws IOException {

        return new TargoBankPdfPageEven();
    }

    /**
     * @throws IOException
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfPageFirst()
     */
    @Override
    protected AbstractBankPDFPage getBankPdfPageFirst() throws IOException {

        return new TargoBankPdfPageFirst();
    }

    /**
     * @throws IOException
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getBankPdfPageOdd()
     */
    @Override
    protected AbstractBankPDFPage getBankPdfPageOdd() throws IOException {

        return new TargoBankPdfPageOdd();
    }

    /**
     * @throws IOException
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getLastPage()
     */
    @Override
    protected int getLastPage() throws IOException {

        int lastPage = this.document.getPages().getCount();
        final PDFTextStripper textStripper = new PDFTextStripper();
        for (int pageNumber = 1; pageNumber <= this.document.getPages().getCount(); pageNumber++) {
            textStripper.setStartPage(pageNumber);
            textStripper.setEndPage(pageNumber);
            // Fetch page text and insert into "pages" string
            final String pageText = textStripper.getText(this.document);
            if (pageText.contains(TargoBankPdfExtractor.LAST_PAGE_TEXT)) {
                lastPage = pageNumber;
                break;
            }
        }
        return lastPage;
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getNumberFormat()
     */
    @Override
    protected NumberFormat getNumberFormat() {

        return TargoBankPdfExtractor.N_FORMAT;
    }

    /**
     * @throws PdfExtractorException
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getRechungsDatum(AbstractBankPDFPage
     *      textAreas)
     */
    @Override
    protected String getRechungsDatum(final AbstractBankPDFPage textAreas) throws PdfExtractorException {

        final String[] parts = textAreas.getTextForRegion(AbstractBankPDFPage.RECHNUNGS_DATUM).split(" "); //$NON-NLS-1$
        if (parts.length != 3) { // NOPMD on 20.06.21, 19:24
            throw new PdfExtractorException("Date length is not 3", PdfExtractorExceptionType.INVALID_INVOICE_DATE); //$NON-NLS-1$
        }
        String datum = StringUtils.EMPTY;
        try {
            final StringBuilder datumBuilder = new StringBuilder(20);
            for (final String part : parts) {
                datumBuilder.append(part.split(AbstractPdfExtractor.WORT_DELIMITER)[2].replace("\n", StringUtils.EMPTY)) //$NON-NLS-1$
                        .append(' ');
            }
            datum = datumBuilder.toString();

            final SimpleDateFormat sdf1 = new SimpleDateFormat("dd. MMM yyyy", Locale.GERMAN); //$NON-NLS-1$
            final SimpleDateFormat sdf2 = new SimpleDateFormat("dd.MM.yyyy", Locale.GERMAN); //$NON-NLS-1$
            datum = sdf2.format(sdf1.parse(datum));
            Logger.trace("PT (rechnungsdatum): " + datum); //$NON-NLS-1$
        } catch (final ParseException e) {
            throw new PdfExtractorException(PdfExtractorExceptionType.INVALID_INVOICE_DATE, e);
        }
        return datum;
    }

    @Override
    protected BigDecimal getSaldo(final AbstractBankPDFPage textAreas) {

        // Saldo is always on the last page
        BigDecimal saldo = null;

        try {

            final PDFTextStripper textStripper = new PDFTextStripper();
            textStripper.setStartPage(this.lastPage);
            textStripper.setEndPage(this.lastPage);
            // Fetch page text and insert into "pages" string
            final String pageText = textStripper.getText(this.document);

            final int posSaldoStart = pageText.indexOf(TargoBankPdfExtractor.LAST_PAGE_TEXT) + 13;
            final int posSaldoEnd = pageText.indexOf(textStripper.getLineSeparator(), posSaldoStart);

            String saldoText = pageText.substring(posSaldoStart, posSaldoEnd);
            saldoText = saldoText.replace(".", "").replace(",", "."); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$

            Logger.debug(String.format("Saldo in page: %s", saldoText)); //$NON-NLS-1$
            saldo = new BigDecimal(saldoText);

        } catch (final IOException e) {
            Logger.warn("Saldo nicht gefunden. Fehler: " + e.getMessage()); //$NON-NLS-1$
        }

        return saldo;

    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#getTransactionCreator()
     */
    @Override
    protected AbstractTransactionCreator getTransactionCreator() {

        return new TargoBankTransactionCreator(this.buchungsDaten, TargoBankPdfExtractor.LINE_HEIGH);
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotations(java.util.List)
     */
    @Override
    protected void removeAnnotations(final List<PdfEintrag> pdfWortList) {

        final double yOfFirstDatum = this.buchungsDaten.get(this.buchungsDaten.size() - 1).getPositionY() - 10;
        Logger.debug("First: " + yOfFirstDatum); //$NON-NLS-1$
        for (int i = pdfWortList.size() - 1; i >= 0; i--) {
            Logger.debug("Check: " + pdfWortList.get(i).getPositionY()); //$NON-NLS-1$
            if (pdfWortList.get(i).getPositionY() < yOfFirstDatum) {
                Logger.debug("Remove"); //$NON-NLS-1$
                pdfWortList.remove(i);
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#removeAnnotationsBookingDates(java.util.List)
     */
    @Override
    protected void removeAnnotationsBookingDates(final List<PdfEintrag> buchungenDatum) {

        if (!buchungenDatum.isEmpty()) {
            final double xOfFirstDatum = buchungenDatum.get(0).getPositionX();
            Logger.trace("First date: " + xOfFirstDatum); //$NON-NLS-1$
            for (int i = buchungenDatum.size() - 1; i > 0; i--) {
                Logger.trace("Check date x position: " + buchungenDatum.get(i).getPositionX()); //$NON-NLS-1$
                if (Math.abs(buchungenDatum.get(i).getPositionX() - xOfFirstDatum) > Utils.DOUBLE_TREAT_AS_EQUAL) {
                    Logger.trace("Remove date as it is an annotation in the bottom"); //$NON-NLS-1$
                    buchungenDatum.remove(i);
                }
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.AbstractPdfExtractor#reOrder(java.util.List)
     */
    @Override
    protected void reOrder(final List<PdfBuchung> buchungen) {

        // Leider mssen wir richtig sortieren, da die Umstze nach Karten
        // sortiert sind
        buchungen.sort((final PdfBuchung buchung1,
                final PdfBuchung buchung2) -> buchung1.getBuchungsDatum().compareTo(buchung2.getBuchungsDatum()) * -1);
    }

}
