/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.messages;

import java.rmi.RemoteException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.apache.commons.lang.StringUtils;

import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.WebAccountInformation;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.datasource.rmi.DBIterator;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.HBCIDBService;
import de.willuhn.jameica.hbci.rmi.Nachricht;
import de.willuhn.jameica.messaging.StatusBarMessage;
import de.willuhn.jameica.system.Application;
import de.willuhn.logging.Logger;
import de.willuhn.util.ApplicationException;

/**
 * Abstrakte-Klasse die Bank-Nachrichten verarbeitet
 *
 * @author Frank Mnster
 */
public abstract class AbstractBankMessages<C, I> implements InterfaceBankMessages {

    /**
     * Logger information in case an method is not yet implemented
     */
    public static final String NOT_IMPLEMENTED = "Noch nicht implementiert."; //$NON-NLS-1$

    /**
     * Liste mit Bank-Nachrichten
     */
    protected List<MahoeMessage> messages;

    /**
     * Der WebClient
     */
    protected C webclient;

    /**
     * WebAccountInfo for SantanderBank account
     */
    protected final I accountInfo;

    /**
     * berprft, ob die Nachricht schon existiert.
     *
     * @param msg
     *            - a {@link MahoeMessage} object
     * @return true, wenn die Nachricht bereits vorhanden ist, false sonst.
     */
    public static boolean existsMessage(final MahoeMessage msg) {

        return AbstractBankMessages.existsMessage(msg.getBlz(), msg.getDate(), msg.getSubject());

    }

    /**
     * berprft, ob die Nachricht schon existiert.
     *
     * @param blz
     * @param date
     * @param subject
     * @return true, wenn die Nachricht bereits vorhanden ist, false sonst.
     */
    public static boolean existsMessage(final String blz, final Date date, final String subject) {

        final String keyToTest = MahoeMessage.createDuplicationKey(date, subject);
        Logger.debug("Key der neuen Nachricht: " + keyToTest); //$NON-NLS-1$
        boolean exist = false;
        try {
            final DBIterator<Nachricht> messages = AbstractBankMessages.getBankMessages(blz);
            while (messages.hasNext()) {
                final Nachricht msg = messages.next();
                final String keyFormDb = msg.getMeta("doubleKey", StringUtils.EMPTY); //$NON-NLS-1$
                Logger.debug("Keyvergleich, neue Nachricht/gespeicherte Nachricht:\n" + keyToTest + "\n" + keyFormDb); //$NON-NLS-1$ //$NON-NLS-2$
                if (keyFormDb.equals(keyToTest)) {
                    exist = true;
                    break;
                }
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractBankMessages.compareFailed"), e); //$NON-NLS-1$
        }
        return exist;
    }

    /**
     * Liefert die Nachrichtenliste fr eine Bank
     *
     * @param blz
     *            die Bankleitzahl
     * @return die Liste mit den Nachrichten
     * @throws RemoteException
     */
    public static DBIterator<Nachricht> getBankMessages(final String blz) throws RemoteException {

        final HBCIDBService service = Settings.getDBService();
        final DBIterator<Nachricht> list = service.createList(Nachricht.class);
        list.addFilter("blz=?", blz); //$NON-NLS-1$
        return list;
    }

    /**
     * Konstruktor mit Angabe des initialisierten WebClient
     *
     * @param webClient
     */
    protected AbstractBankMessages(final C webClient, final I accountInfo) {
        this.webclient = webClient;
        this.accountInfo = accountInfo;
        this.messages = new ArrayList<>();
    }

    /**
     * Returns the date of the latest message of the bank
     *
     * @param accountInfo
     *            with the bank (BLZ) to check
     * @return the date of the latest message or 01-01-2000
     */
    public Date getLastMsgDate(final WebAccountInformation accountInfo) {

        LocalDate lastMsgDate = LocalDate.of(2000, 1, 1);

        String blz;
        try {
            blz = accountInfo.getKonto().getBLZ();
            final DBIterator<Nachricht> bankMsgs = AbstractBankMessages.getBankMessages(blz);
            while (bankMsgs.hasNext()) {
                final Nachricht msg = bankMsgs.next();
                final LocalDate date = Utils.dateToLocalDate(msg.getDatum());
                if (date.isAfter(lastMsgDate)) {
                    lastMsgDate = date;
                }
            }
        } catch (final RemoteException e) {
            Logger.error(I18N.tr("AbstractBankMessages.erreRetrievingLastMsgDate", e.getLocalizedMessage()), e); //$NON-NLS-1$
        }
        return Utils.localDateToDate(lastMsgDate);
    }

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages# readMessages
     *      (java.util.Date)
     * @throws WebSiteException
     * @throws ApplicationException
     */
    @Override
    public abstract void readMessages(Date startDate) throws WebSiteException, ApplicationException;

    /**
     * @see de.frankmuenster.mahoe.messages.InterfaceBankMessages# saveMessage
     *      ()
     */
    @Override
    public int saveMessage() throws ApplicationException {

        int savedMsg = 0;

        for (final MahoeMessage mahoeMsg : this.messages) {
            try {
                if (!AbstractBankMessages.existsMessage(mahoeMsg)) {
                    mahoeMsg.saveAsNachricht();
                }
                this.markAsRead(mahoeMsg);

                final String text = I18N.tr("AbstractBankMessages.newBankMessage"); //$NON-NLS-1$
                Application.getMessagingFactory()
                        .sendMessage(new StatusBarMessage(text, StatusBarMessage.TYPE_SUCCESS)); // NOPMD
                savedMsg++;
            } catch (final RemoteException e) {
                Logger.error(I18N.tr("AbstractBankMessages.storeFailed"), e); //$NON-NLS-1$
                // Im Fehlerfall zeigen wir einfach den Dialog an
                try {
                    Application.getCallback().notifyUser(mahoeMsg.getMessage());
                } catch (final Exception e1) { // NOPMD
                    throw new ApplicationException(I18N.tr("AbstractBankMessages.viewFailed"), e1); // NOPMD //$NON-NLS-1$
                }
            }
        }

        return savedMsg;
    }

    /**
     * Add a message , if is does not exists already
     *
     * @param msg
     * @return
     */
    protected void addMessage(final MahoeMessage msg) {

        if (AbstractBankMessages.existsMessage(msg)) {
            Logger.info("Message exists already. " + msg.getBlz() + "-" + msg.getDate() + "-" + msg.getSubject());
        } else {
            Logger.info("Adding message: " + msg.getBlz() + "-" + msg.getDate() + "-" + msg.getSubject());
            this.messages.add(msg);
        }

    }
}
