
package de.frankmuenster.mahoe.pdfextractor;

import java.math.BigDecimal;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.frankmuenster.mahoe.utils.I18N;
import de.willuhn.logging.Logger;

/**
 * Erstellt eine Liste mit Hibiscus-Umstzen.
 *
 * @author Frank Mnster
 */
public abstract class AbstractTransactionCreator implements TransactionsCreator {

    /**
     * Die Liste mit den Buchungen
     */
    protected final List<PdfBuchung> transactions;

    /**
     * Liste mit den Positionen der einzelnen Zeilen
     */
    protected final Map<Long, Integer> taNumbers;

    /**
     * Der Datumsformatierer
     */
    protected final SimpleDateFormat sdf;

    /**
     * Dezimalformatierer
     */
    private final NumberFormat numFormat;

    /**
     * Y-Position der ersten Buchung auf der Seite.
     */
    private final double firstLineY;

    /**
     * Die Zeilenhhe
     */
    protected final double lineHeigh;

    /**
     * Konstruktor mit Angabe der Buchungsdatums-Liste. Daran werden dann alle
     * anderen Zeilen anhand ihrer Hhe gemessen.
     *
     * @param buchungsDaten
     *            die Liste mit den Buchungsdaten
     * @param lineHeigh
     *            die Zeilenhhe. Wird bentigt, um die Buchungszeilnummer zu
     *            berechnen.
     */
    protected AbstractTransactionCreator(final List<PdfEintrag> buchungsDaten, final double lineHeigh) {
        this.sdf = this.getDateFormat(); // NOPMD on 15.06.25, 19:57
        this.numFormat = this.getNumberFormat(); // NOPMD on 15.06.25, 19:57
        this.transactions = new ArrayList<>();
        this.taNumbers = new HashMap<>();
        this.lineHeigh = lineHeigh;
        this.firstLineY = buchungsDaten.get(0).getPositionY();
        Date bDatum = null;
        int taNumber = 0;
        for (final PdfEintrag buchungsDatum : buchungsDaten) {
            try {
                bDatum = this.sdf.parse(buchungsDatum.getWort());
                final PdfBuchung buchung = new PdfBuchung(); // NOPMD
                buchung.setBuchungsDatum(bDatum);
                this.transactions.add(buchung);
                final long buchungsZeile = Math.round((this.firstLineY - buchungsDatum.getPositionY()) / lineHeigh);
                this.taNumbers.put(buchungsZeile, taNumber++);
            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.bookingDateParseError", e.getMessage())); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addBetraege(java.util.List)
     */
    @Override
    public void addBetraege(final List<PdfEintrag> values) {

        BigDecimal value = null;
        for (final PdfEintrag valuesWord : values) {
            try {
                value = new BigDecimal(this.numFormat.parse(valuesWord.getWort()).toString()); // NOPMD
                final Integer row = this.getRow(valuesWord.getPositionY());
                if (row != null) {
                    final PdfBuchung transaction = this.transactions.get(row);
                    transaction.setBetrag(value);
                }
            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.valueParseError") + e.getMessage()); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addDescriptions(java.util.List)
     */
    @Override
    public void addDescriptions(final List<PdfEintrag> describtions) {

        for (final PdfEintrag description : describtions) {
            final Integer row = this.getRow(description.getPositionY());
            if (row != null) {
                final PdfBuchung transaction = this.transactions.get(row);
                if (transaction.getZweck().length() == 0) {
                    transaction.setZweck(description.getWort());
                } else {
                    transaction.setZweck(transaction.getZweck() + " " + description.getWort()); //$NON-NLS-1$
                }
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addExchangeRates(java.util.List)
     */
    @Override
    public void addExchangeRates(final List<PdfEintrag> exchangeRates) {

        BigDecimal rate = null;
        for (final PdfEintrag rateWord : exchangeRates) {
            try {
                rate = new BigDecimal(this.numFormat.parse(rateWord.getWort()).toString()); // NOPMD
                final Integer row = this.getRow(rateWord.getPositionY());
                if (row != null) {
                    final PdfBuchung transaction = this.transactions.get(row);
                    transaction.setKurs(rate);
                }
            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.exchangeRateParseError", e.getMessage())); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addFcValues(java.util.List)
     */
    @Override
    public void addFcValues(final List<PdfEintrag> fcValues) {

        BigDecimal fcValue;
        for (final PdfEintrag fcValueWord : fcValues) {
            try {
                fcValue = new BigDecimal(this.numFormat.parse(fcValueWord.getWort()).toString()); // NOPMD
                final Integer row = this.getRow(fcValueWord.getPositionY());
                if (row != null) {
                    final PdfBuchung transaction = this.transactions.get(row);
                    transaction.setFremdWaehrungsBetrag(fcValue);
                }
            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.foreignCurrencyParseError", e.getMessage())); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addForeignCurrencys(java.util.List)
     */
    @Override
    public void addForeignCurrencys(final List<PdfEintrag> foreignCurrencys) {

        for (final PdfEintrag foreignCurrency : foreignCurrencys) {
            final Integer row = this.getRow(foreignCurrency.getPositionY());
            if (row != null) {
                final PdfBuchung transaction = this.transactions.get(row);
                transaction.setFremdWaehrung(foreignCurrency.getWort());
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#addKaufDaten(java.util.List)
     */
    @Override
    public void addKaufDaten(final List<PdfEintrag> valutas) {

        Date vDatum = null;
        for (final PdfEintrag valutaDatum : valutas) {
            try {
                synchronized (AbstractTransactionCreator.class) {
                    vDatum = this.sdf.parse(valutaDatum.getWort());
                }
                final Integer number = this.getRow(valutaDatum.getPositionY());
                if (number != null) {
                    final PdfBuchung transaction = this.transactions.get(number);
                    transaction.setKaufDatum(vDatum);
                }
            } catch (final ParseException e) {
                Logger.error(I18N.tr("AbstractTransactionCreator.valutaParseError", e.getMessage())); //$NON-NLS-1$
            }
        }
    }

    /**
     * @see de.frankmuenster.mahoe.pdfextractor.TransactionsCreator#getTansactions()
     */
    @Override
    public List<PdfBuchung> getTansactions() {

        return this.transactions;
    }

    /**
     * Liefert das Datumsformat in dem die Daten verarbeitet werden sollen. Muss
     * von der Unterklasse implementiert werden.
     *
     * @return das Datumsformt der Kreditkartenausz&uuml;ge
     */
    protected abstract SimpleDateFormat getDateFormat();

    /**
     * Liefert das NumberFormat in dem die Betr&auml;ge verarbeitet werden
     * sollen. Muss von der Unterklasse implementiert werden.
     *
     * @return das Zahlenformt der Kreditkartenausz&uuml;ge
     */
    protected abstract NumberFormat getNumberFormat();

    /**
     * Liefert die Zeilenummer zu einer Y-Position
     *
     * @param posY
     *            die y-Position
     * @return die Zeilenummer
     */
    protected Integer getRow(final double posY) {

        final long myRow = Math.round((this.firstLineY - posY) / this.lineHeigh);
        return this.taNumbers.get(myRow);
    }
}
