/*******************************************************************************
 * Copyright 2015 - 2019 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.mahoe.statements;

import java.rmi.RemoteException;
import java.time.LocalDate;
import java.time.format.DateTimeParseException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;

import org.kapott.hbci.GV_Result.GVRKontoauszug.Format;

import de.frankmuenster.mahoe.MahoeSynchronizeBackend;
import de.frankmuenster.mahoe.utils.I18N;
import de.frankmuenster.mahoe.utils.Utils;
import de.frankmuenster.mahoe.web.MahoeKontoauszugPDF;
import de.frankmuenster.mahoe.web.WebAccountInformation;
import de.frankmuenster.mahoe.web.WebSiteException;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.jameica.hbci.MetaKey;
import de.willuhn.jameica.hbci.Settings;
import de.willuhn.jameica.hbci.rmi.Konto;
import de.willuhn.jameica.hbci.rmi.Kontoauszug;
import de.willuhn.jameica.hbci.server.KontoauszugPdfUtil;
import de.willuhn.logging.Logger;

/**
 * The abstract class for statement finders. Must be extended by bank specific
 * implementations. *
 *
 * @author Frank Mnster
 * @param <C>
 *            the web client either WebClient from HTMLUnit or WebDriver from
 *            Selenium
 * @param <I>
 *            the concrete implementation of {@link WebAccountInformation}
 */
public abstract class AbstractAccountStatementFinder<C, I extends WebAccountInformation> {

    /**
     * The Santander account info
     */
    protected final I accountInfo;

    /**
     * The webclient. needs to be initialised and logged in
     */
    protected final C webClient;

    /**
     * List of found account statements
     */
    private final List<MahoeKontoauszugPDF> accountStatements = new ArrayList<>();

    /**
     * Date of last saved account statement
     */
    protected final Date lastStatementDate;

    /**
     * Search run already
     */
    private boolean searchExecuted;

    /**
     * Constructs a new Finder for account statements. Must be called from real
     * implementations.
     *
     * @param webClient
     * @param accountInfo
     */
    public AbstractAccountStatementFinder(final C webClient, final I accountInfo) {

        this.accountInfo = accountInfo;
        this.webClient = webClient;

        this.lastStatementDate = this.getLastDownloadDate();
    }

    /**
     * Start looking for new sheets based on the
     * {@link MetaKey#KONTOAUSZUG_INTERVAL_LAST}
     *
     * @throws WebSiteException
     */
    public void findStatements() throws WebSiteException {

        this.startSearch();
        this.searchExecuted = true;

    }

    /**
     * Return the found sheets or an empty list in case search was not yet
     * started or nothing was found.
     *
     * @return the list with {@link MahoeKontoauszugPDF}
     */
    public List<MahoeKontoauszugPDF> getSheets() {

        return this.accountStatements;
    }

    /**
     * @return the searchExecuted
     */
    public boolean isSearchExecuted() {

        return this.searchExecuted;
    }

    /**
     * Creates a new {@link Kontoauszug} and then a new
     * {@link MahoeKontoauszugPDF} with the given information. Finally it is
     * added to the internal account statement list.
     *
     * @param statementInfo
     *            the basic information about the statement
     * @param data
     *            the PDF as byte[]
     * @throws RemoteException
     *             in case the {@link Kontoauszug} could not be created.
     */
    protected void addStatement(final MahoeAccountStatement statementInfo, final byte[] data) throws RemoteException {

        if (!statementInfo.isComplete()) {
            throw new RemoteException(I18N.tr("AbstractAccountStatementFinder.statementInfoIncomplete")); //$NON-NLS-1$
        }
        final Kontoauszug auszug = Settings.getDBService().createObject(Kontoauszug.class, null);

        auszug.setErstellungsdatum(statementInfo.getCreateDate());
        auszug.setFormat(Format.PDF.getCode());

        final Calendar cal = new GregorianCalendar();
        cal.setTime(statementInfo.getCreateDate());

        auszug.setJahr(statementInfo.getYear());
        auszug.setNummer(statementInfo.getNumber());

        auszug.setKonto(this.accountInfo.getKonto());
        auszug.setName1(statementInfo.getName1());
        auszug.setName2(statementInfo.getName2());
        auszug.setName3(statementInfo.getName3());
        auszug.setKommentar(statementInfo.getComment());

        final MahoeKontoauszugPDF mahoeAuszug = new MahoeKontoauszugPDF(auszug, data);
        this.accountStatements.add(mahoeAuszug);

    }

    /**
     * Starts the search of new account statements
     *
     * @throws WebSiteException
     */
    protected abstract void startSearch() throws WebSiteException;

    /**
     * Get the last download date from {@link MetaKey#KONTOAUSZUG_INTERVAL_LAST}
     * minus the days configured in the settings
     *
     * @return last download day minus the days configured in the settings.
     */
    private Date getLastDownloadDate() {

        Date date = new Date(0);

        final Konto kto = this.accountInfo.getKonto();
        if (kto != null) {

            try {
                // get statement list from account
                final GenericIterator<
                        Kontoauszug> existingStatemets = KontoauszugPdfUtil.getList(kto, null, null, false, true);

                Date statementDate = new Date(0);
                while (existingStatemets.hasNext()) {
                    final Kontoauszug storedStatemant = existingStatemets.next();
                    final Date tmpDate = storedStatemant.getErstellungsdatum();
                    if (tmpDate.after(statementDate)) {
                        statementDate = tmpDate;
                    }
                }
                if (statementDate != null) {
                    date = statementDate;
                }

                final String dateString = MetaKey.KONTOAUSZUG_INTERVAL_LAST.get(kto);
                final int downloadPeriod = Integer
                        .parseInt(kto.getMeta(MahoeSynchronizeBackend.ACC_STATEM_MAX_DAYS_BACK, "90")); //$NON-NLS-1$
                if (dateString != null) {
                    LocalDate localdate = Utils.convertTextToLocalDate(dateString, "dd.MM.yyyy HH:mm"); //$NON-NLS-1$
                    localdate = localdate.minusDays(downloadPeriod);
                    final Date lastDownLoadDate = Utils.localDateToDate(localdate);
                    if (lastDownLoadDate.before(date)) {
                        date = lastDownLoadDate;
                    }
                }

            } catch (RemoteException | DateTimeParseException e) {
                Logger.error(I18N.tr("AccountStatementFinder.errorGetingLastDownloadDate"), e); //$NON-NLS-1$
            }
        }
        return date;
    }

}
